@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2009-2011 Roger Light <roger@atchoo.org>
+Copyright (c) 2009-2012 Roger Light <roger@atchoo.org>
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -28,6 +28,7 @@ POSSIBILITY OF SUCH DAMAGE.
 */
 
 
+#include <errno.h>
 #include <fcntl.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -36,6 +37,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <unistd.h>
 #else
 #include <process.h>
+#include <winsock2.h>
 #define snprintf sprintf_s
 #endif
 
@@ -85,7 +87,25 @@ void my_connect_callback(void *obj, int result)
 				break;
 		}
 		if(rc){
-			if(!quiet) fprintf(stderr, "Error: Publish returned %d, disconnecting.\n", rc);
+			if(!quiet){
+				switch(rc){
+					case MOSQ_ERR_INVAL:
+						fprintf(stderr, "Error: Invalid input. Does your topic contain '+' or '#'?\n");
+						break;
+					case MOSQ_ERR_NOMEM:
+						fprintf(stderr, "Error: Out of memory when trying to publish message.\n");
+						break;
+					case MOSQ_ERR_NO_CONN:
+						fprintf(stderr, "Error: Client not connected when trying to publish.\n");
+						break;
+					case MOSQ_ERR_PROTOCOL:
+						fprintf(stderr, "Error: Protocol error when communicating with broker.\n");
+						break;
+					case MOSQ_ERR_PAYLOAD_SIZE:
+						fprintf(stderr, "Error: Message payload is too large.\n");
+						break;
+				}
+			}
 			mosquitto_disconnect(mosq);
 		}
 	}else{
@@ -241,6 +261,8 @@ int main(int argc, char *argv[])
 	struct mosquitto *mosq = NULL;
 	int rc;
 	int rc2;
+	char hostname[21];
+	char err[1024];
 
 	uint8_t *will_payload = NULL;
 	long will_payloadlen = 0;
@@ -459,7 +481,9 @@ int main(int argc, char *argv[])
 			if(!quiet) fprintf(stderr, "Error: Out of memory.\n");
 			return 1;
 		}
-		snprintf(id, 30, "mosquitto_pub_%d", getpid());
+		memset(hostname, 0, 21);
+		gethostname(hostname, 20);
+		snprintf(id, 23, "mosq_pub_%d_%s", getpid(), hostname);
 	}
 
 	if(!topic || mode == MSGMODE_NONE){
@@ -506,7 +530,18 @@ int main(int argc, char *argv[])
 
 	rc = mosquitto_connect(mosq, host, port, keepalive, true);
 	if(rc){
-		if(!quiet) fprintf(stderr, "Unable to connect (%d).\n", rc);
+		if(!quiet){
+			if(rc == MOSQ_ERR_ERRNO){
+#ifndef WIN32
+				strerror_r(errno, err, 1024);
+#else
+				FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, NULL, errno, 0, (LPTSTR)&err, 1024, NULL);
+#endif
+				fprintf(stderr, "Error: %s\n", err);
+			}else{
+				fprintf(stderr, "Unable to connect (%d).\n", rc);
+			}
+		}
 		return rc;
 	}
 
