@@ -29,7 +29,6 @@ POSSIBILITY OF SUCH DAMAGE.
 #ifndef _NET_MOSQ_H_
 #define _NET_MOSQ_H_
 
-#include <stdint.h>
 #ifndef WIN32
 #include <unistd.h>
 #else
@@ -40,6 +39,16 @@ typedef int ssize_t;
 #include <mosquitto_internal.h>
 #include <mosquitto.h>
 
+#ifdef WITH_BROKER
+struct _mosquitto_db;
+#endif
+
+#ifdef WIN32
+#  define COMPAT_CLOSE(a) closesocket(a)
+#else
+#  define COMPAT_CLOSE(a) close(a)
+#endif
+
 /* For when not using winsock libraries. */
 #ifndef INVALID_SOCKET
 #define INVALID_SOCKET -1
@@ -53,9 +62,9 @@ void _mosquitto_net_init(void);
 void _mosquitto_net_cleanup(void);
 
 void _mosquitto_packet_cleanup(struct _mosquitto_packet *packet);
-void _mosquitto_packet_queue(struct _mosquitto_core *core, struct _mosquitto_packet *packet);
-int _mosquitto_socket_connect(struct _mosquitto_core *core, const char *host, uint16_t port);
-int _mosquitto_socket_close(struct _mosquitto_core *core);
+int _mosquitto_packet_queue(struct mosquitto *mosq, struct _mosquitto_packet *packet);
+int _mosquitto_socket_connect(struct mosquitto *mosq, const char *host, uint16_t port);
+int _mosquitto_socket_close(struct mosquitto *mosq);
 
 int _mosquitto_read_byte(struct _mosquitto_packet *packet, uint8_t *byte);
 int _mosquitto_read_bytes(struct _mosquitto_packet *packet, uint8_t *bytes, uint32_t count);
@@ -67,7 +76,14 @@ void _mosquitto_write_bytes(struct _mosquitto_packet *packet, const uint8_t *byt
 void _mosquitto_write_string(struct _mosquitto_packet *packet, const char *str, uint16_t length);
 void _mosquitto_write_uint16(struct _mosquitto_packet *packet, uint16_t word);
 
-ssize_t _mosquitto_net_read(struct _mosquitto_core *core, void *buf, size_t count);
-ssize_t _mosquitto_net_write(struct _mosquitto_core *core, void *buf, size_t count);
+ssize_t _mosquitto_net_read(struct mosquitto *mosq, void *buf, size_t count);
+ssize_t _mosquitto_net_write(struct mosquitto *mosq, void *buf, size_t count);
+
+int _mosquitto_packet_write(struct mosquitto *mosq);
+#ifdef WITH_BROKER
+int _mosquitto_packet_read(struct _mosquitto_db *db, int context_index);
+#else
+int _mosquitto_packet_read(struct mosquitto *mosq);
+#endif
 
 #endif
