@@ -53,7 +53,9 @@ int _mosquitto_handle_connack(struct mosquitto *mosq)
 	rc = _mosquitto_read_byte(&mosq->in_packet, &result);
 	if(rc) return rc;
 	if(mosq->on_connect){
+		mosq->in_callback = true;
 		mosq->on_connect(mosq->obj, result);
+		mosq->in_callback = false;
 	}
 	switch(result){
 		case 0:
