@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2009-2011 Roger Light <roger@atchoo.org>
+Copyright (c) 2009-2012 Roger Light <roger@atchoo.org>
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -79,6 +79,17 @@ int _mosquitto_packet_alloc(struct _mosquitto_packet *packet)
 void _mosquitto_check_keepalive(struct mosquitto *mosq)
 {
 	assert(mosq);
+#if defined(WITH_BROKER) && defined(WITH_BRIDGE)
+	/* Check if a lazy bridge should be timed out due to idle. */
+	if(mosq->bridge && mosq->bridge->start_type == bst_lazy
+				&& mosq->sock != INVALID_SOCKET
+				&& time(NULL) - mosq->last_msg_out >= mosq->bridge->idle_timeout){
+
+		_mosquitto_log_printf(NULL, MOSQ_LOG_NOTICE, "Bridge connection %s has exceeded idle timeout, disconnecting.", mosq->id);
+		_mosquitto_socket_close(mosq);
+		return;
+	}
+#endif
 	if(mosq->sock != INVALID_SOCKET && time(NULL) - mosq->last_msg_out >= mosq->keepalive){
 		if(mosq->state == mosq_cs_connected){
 			_mosquitto_send_pingreq(mosq);
@@ -138,14 +149,21 @@ uint16_t _mosquitto_mid_generate(struct mosquitto *mosq)
 	return mosq->last_mid;
 }
 
-/* Search for + or # in a string. Return true if found. */
-bool _mosquitto_wildcard_check(const char *str)
+/* Search for + or # in a topic. Return MOSQ_ERR_INVAL if found.
+ * Also returns MOSQ_ERR_INVAL if the topic string is too long.
+ * Returns MOSQ_ERR_SUCCESS if everything is fine.
+ */
+int _mosquitto_topic_wildcard_len_check(const char *str)
 {
+	int len = 0;
 	while(str && str[0]){
 		if(str[0] == '+' || str[0] == '#'){
-			return true;
+			return MOSQ_ERR_INVAL;
 		}
+		len++;
 		str = &str[1];
 	}
-	return false;
+	if(len > 65535) return MOSQ_ERR_INVAL;
+
+	return MOSQ_ERR_SUCCESS;
 }
