@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2009-2011 Roger Light <roger@atchoo.org>
+Copyright (c) 2009-2012 Roger Light <roger@atchoo.org>
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -122,6 +122,12 @@ int mqtt3_bridge_connect(mosquitto_db *db, struct mosquitto *context)
 	context->in_packet.payload = NULL;
 	mqtt3_bridge_packet_cleanup(context);
 
+	for(i=0; i<context->bridge->topic_count; i++){
+		if(context->bridge->topics[i].direction == bd_out || context->bridge->topics[i].direction == bd_both){
+			if(mqtt3_sub_add(context, context->bridge->topics[i].topic, context->bridge->topics[i].qos, &db->subs)) return 1;
+		}
+	}
+
 	_mosquitto_log_printf(NULL, MOSQ_LOG_NOTICE, "Connecting bridge %s", context->bridge->name);
 	rc = _mosquitto_socket_connect(context, context->bridge->address, context->bridge->port);
 	if(rc != MOSQ_ERR_SUCCESS){
@@ -139,23 +145,14 @@ int mqtt3_bridge_connect(mosquitto_db *db, struct mosquitto *context)
 		notification_payload[1] = '\0';
 		mqtt3_db_messages_easy_queue(db, context, notification_topic, 1, 2, (uint8_t *)&notification_payload, 1);
 		rc = _mosquitto_will_set(context, true, notification_topic, 2, (uint8_t *)&notification_payload, 1, true);
-		if(rc){
+		if(rc != MOSQ_ERR_SUCCESS){
 			_mosquitto_free(notification_topic);
 			return rc;
 		}
 		_mosquitto_free(notification_topic);
 	}
 
-	rc = _mosquitto_send_connect(context, context->keepalive, context->clean_session);
-	if(rc) return rc;
-
-	for(i=0; i<context->bridge->topic_count; i++){
-		if(context->bridge->topics[i].direction == bd_out || context->bridge->topics[i].direction == bd_both){
-			if(mqtt3_sub_add(context, context->bridge->topics[i].topic, context->bridge->topics[i].qos, &db->subs)) return 1;
-		}
-	}
-
-	return MOSQ_ERR_SUCCESS;
+	return _mosquitto_send_connect(context, context->keepalive, context->clean_session);
 }
 
 void mqtt3_bridge_packet_cleanup(struct mosquitto *context)
