@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2009-2011 Roger Light <roger@atchoo.org>
+Copyright (c) 2009-2012 Roger Light <roger@atchoo.org>
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -165,7 +165,7 @@ int main(int argc, char *argv[])
 
 	mqtt3_config_init(&config);
 	rc = mqtt3_config_parse_args(&config, argc, argv);
-	if(rc) return rc;
+	if(rc != MOSQ_ERR_SUCCESS) return rc;
 	int_db.config = &config;
 
 	if(config.daemon){
@@ -195,10 +195,10 @@ int main(int argc, char *argv[])
 		}
 	}
 	rc = drop_privileges(&config);
-	if(rc) return rc;
+	if(rc != MOSQ_ERR_SUCCESS) return rc;
 
 	rc = mqtt3_db_open(&config, &int_db);
-	if(rc){
+	if(rc != MOSQ_ERR_SUCCESS){
 		_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Couldn't open database.");
 		return rc;
 	}
@@ -216,7 +216,7 @@ int main(int argc, char *argv[])
 	mqtt3_db_messages_easy_queue(&int_db, NULL, "$SYS/broker/version", 2, strlen(buf), (uint8_t *)buf, 1);
 	snprintf(buf, 1024, "%s", TIMESTAMP);
 	mqtt3_db_messages_easy_queue(&int_db, NULL, "$SYS/broker/timestamp", 2, strlen(buf), (uint8_t *)buf, 1);
-	snprintf(buf, 1024, "%s", "$Revision: 5e732f3db18b $"); // Requires hg keyword extension.
+	snprintf(buf, 1024, "%s", "$Revision: e745e1ab5007 $"); // Requires hg keyword extension.
 	mqtt3_db_messages_easy_queue(&int_db, NULL, "$SYS/broker/changeset", 2, strlen(buf), (uint8_t *)buf, 1);
 
 	listener_max = -1;
