@@ -31,7 +31,9 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #ifdef WITH_PERSISTENCE
 
+#ifndef WIN32
 #include <arpa/inet.h>
+#endif
 #include <assert.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -45,22 +47,22 @@ POSSIBILITY OF SUCH DAMAGE.
 
 static int _db_restore_sub(mosquitto_db *db, const char *client_id, const char *sub, int qos);
 
-static mqtt3_context *_db_find_or_add_context(mosquitto_db *db, const char *client_id, uint16_t last_mid)
+static struct mosquitto *_db_find_or_add_context(mosquitto_db *db, const char *client_id, uint16_t last_mid)
 {
-	mqtt3_context *context;
-	mqtt3_context **tmp_contexts;
+	struct mosquitto *context;
+	struct mosquitto **tmp_contexts;
 	int i;
 
 	context = NULL;
 	for(i=0; i<db->context_count; i++){
-		if(db->contexts[i] && !strcmp(db->contexts[i]->core.id, client_id)){
+		if(db->contexts[i] && !strcmp(db->contexts[i]->id, client_id)){
 			context = db->contexts[i];
 			break;
 		}
 	}
 	if(!context){
 		context = mqtt3_context_init(-1);
-		context->core.clean_session = false;
+		context->clean_session = false;
 
 		for(i=0; i<db->context_count; i++){
 			if(!db->contexts[i]){
@@ -70,7 +72,7 @@ static mqtt3_context *_db_find_or_add_context(mosquitto_db *db, const char *clie
 		}
 		if(i==db->context_count){
 			db->context_count++;
-			tmp_contexts = _mosquitto_realloc(db->contexts, sizeof(mqtt3_context*)*db->context_count);
+			tmp_contexts = _mosquitto_realloc(db->contexts, sizeof(struct mosquitto*)*db->context_count);
 			if(tmp_contexts){
 				db->contexts = tmp_contexts;
 				db->contexts[db->context_count-1] = context;
@@ -78,15 +80,15 @@ static mqtt3_context *_db_find_or_add_context(mosquitto_db *db, const char *clie
 				return NULL;
 			}
 		}
-		context->core.id = _mosquitto_strdup(client_id);
+		context->id = _mosquitto_strdup(client_id);
 	}
 	if(last_mid){
-		context->core.last_mid = last_mid;
+		context->last_mid = last_mid;
 	}
 	return context;
 }
 
-static int mqtt3_db_client_messages_write(mosquitto_db *db, int db_fd, mqtt3_context *context)
+static int mqtt3_db_client_messages_write(mosquitto_db *db, FILE *db_fptr, struct mosquitto *context)
 {
 	uint32_t length;
 	dbid_t i64temp;
@@ -95,57 +97,57 @@ static int mqtt3_db_client_messages_write(mosquitto_db *db, int db_fd, mqtt3_con
 	mosquitto_client_msg *cmsg;
 
 	assert(db);
-	assert(db_fd >= 0);
+	assert(db_fptr);
 	assert(context);
 
 	cmsg = context->msgs;
 	while(cmsg){
-		slen = strlen(context->core.id);
+		slen = strlen(context->id);
 
 		length = htonl(sizeof(dbid_t) + sizeof(uint16_t) + sizeof(uint8_t) +
 				sizeof(uint8_t) + sizeof(uint8_t) + sizeof(uint8_t) +
 				sizeof(uint8_t) + 2+slen);
 
 		i16temp = htons(DB_CHUNK_CLIENT_MSG);
-		write_e(db_fd, &i16temp, sizeof(uint16_t));
-		write_e(db_fd, &length, sizeof(uint32_t));
+		write_e(db_fptr, &i16temp, sizeof(uint16_t));
+		write_e(db_fptr, &length, sizeof(uint32_t));
 
 		i16temp = htons(slen);
-		write_e(db_fd, &i16temp, sizeof(uint16_t));
-		write_e(db_fd, context->core.id, slen);
+		write_e(db_fptr, &i16temp, sizeof(uint16_t));
+		write_e(db_fptr, context->id, slen);
 
 		i64temp = cmsg->store->db_id;
-		write_e(db_fd, &i64temp, sizeof(dbid_t));
+		write_e(db_fptr, &i64temp, sizeof(dbid_t));
 
 		i16temp = htons(cmsg->mid);
-		write_e(db_fd, &i16temp, sizeof(uint16_t));
+		write_e(db_fptr, &i16temp, sizeof(uint16_t));
 
 		i8temp = (uint8_t )cmsg->qos;
-		write_e(db_fd, &i8temp, sizeof(uint8_t));
+		write_e(db_fptr, &i8temp, sizeof(uint8_t));
 
 		i8temp = (uint8_t )cmsg->retain;
-		write_e(db_fd, &i8temp, sizeof(uint8_t));
+		write_e(db_fptr, &i8temp, sizeof(uint8_t));
 
 		i8temp = (uint8_t )cmsg->direction;
-		write_e(db_fd, &i8temp, sizeof(uint8_t));
+		write_e(db_fptr, &i8temp, sizeof(uint8_t));
 
 		i8temp = (uint8_t )cmsg->state;
-		write_e(db_fd, &i8temp, sizeof(uint8_t));
+		write_e(db_fptr, &i8temp, sizeof(uint8_t));
 
 		i8temp = (uint8_t )cmsg->dup;
-		write_e(db_fd, &i8temp, sizeof(uint8_t));
+		write_e(db_fptr, &i8temp, sizeof(uint8_t));
 
 		cmsg = cmsg->next;
 	}
 
 	return MOSQ_ERR_SUCCESS;
 error:
-	mqtt3_log_printf(MOSQ_LOG_ERR, "Error: %s.", strerror(errno));
+	_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: %s.", strerror(errno));
 	return 1;
 }
 
 
-static int mqtt3_db_message_store_write(mosquitto_db *db, int db_fd)
+static int mqtt3_db_message_store_write(mosquitto_db *db, FILE *db_fptr)
 {
 	uint32_t length;
 	dbid_t i64temp;
@@ -155,7 +157,7 @@ static int mqtt3_db_message_store_write(mosquitto_db *db, int db_fd)
 	struct mosquitto_msg_store *stored;
 
 	assert(db);
-	assert(db_fd >= 0);
+	assert(db_fptr);
 
 	stored = db->msg_store;
 	while(stored){
@@ -165,40 +167,40 @@ static int mqtt3_db_message_store_write(mosquitto_db *db, int db_fd)
 				stored->msg.payloadlen + sizeof(uint8_t) + sizeof(uint8_t));
 
 		i16temp = htons(DB_CHUNK_MSG_STORE);
-		write_e(db_fd, &i16temp, sizeof(uint16_t));
-		write_e(db_fd, &length, sizeof(uint32_t));
+		write_e(db_fptr, &i16temp, sizeof(uint16_t));
+		write_e(db_fptr, &length, sizeof(uint32_t));
 
 		i64temp = stored->db_id;
-		write_e(db_fd, &i64temp, sizeof(dbid_t));
+		write_e(db_fptr, &i64temp, sizeof(dbid_t));
 
 		slen = strlen(stored->source_id);
 		i16temp = htons(slen);
-		write_e(db_fd, &i16temp, sizeof(uint16_t));
+		write_e(db_fptr, &i16temp, sizeof(uint16_t));
 		if(slen){
-			write_e(db_fd, stored->source_id, slen);
+			write_e(db_fptr, stored->source_id, slen);
 		}
 
 		i16temp = htons(stored->source_mid);
-		write_e(db_fd, &i16temp, sizeof(uint16_t));
+		write_e(db_fptr, &i16temp, sizeof(uint16_t));
 
 		i16temp = htons(stored->msg.mid);
-		write_e(db_fd, &i16temp, sizeof(uint16_t));
+		write_e(db_fptr, &i16temp, sizeof(uint16_t));
 
 		slen = strlen(stored->msg.topic);
 		i16temp = htons(slen);
-		write_e(db_fd, &i16temp, sizeof(uint16_t));
-		write_e(db_fd, stored->msg.topic, slen);
+		write_e(db_fptr, &i16temp, sizeof(uint16_t));
+		write_e(db_fptr, stored->msg.topic, slen);
 
 		i8temp = (uint8_t )stored->msg.qos;
-		write_e(db_fd, &i8temp, sizeof(uint8_t));
+		write_e(db_fptr, &i8temp, sizeof(uint8_t));
 
 		i8temp = (uint8_t )stored->msg.retain;
-		write_e(db_fd, &i8temp, sizeof(uint8_t));
+		write_e(db_fptr, &i8temp, sizeof(uint8_t));
 
 		i32temp = htonl(stored->msg.payloadlen);
-		write_e(db_fd, &i32temp, sizeof(uint32_t));
+		write_e(db_fptr, &i32temp, sizeof(uint32_t));
 		if(stored->msg.payloadlen){
-			write_e(db_fd, stored->msg.payload, stored->msg.payloadlen);
+			write_e(db_fptr, stored->msg.payload, stored->msg.payloadlen);
 		}
 
 		stored = stored->next;
@@ -206,47 +208,47 @@ static int mqtt3_db_message_store_write(mosquitto_db *db, int db_fd)
 
 	return MOSQ_ERR_SUCCESS;
 error:
-	mqtt3_log_printf(MOSQ_LOG_ERR, "Error: %s.", strerror(errno));
+	_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: %s.", strerror(errno));
 	return 1;
 }
 
-static int mqtt3_db_client_write(mosquitto_db *db, int db_fd)
+static int mqtt3_db_client_write(mosquitto_db *db, FILE *db_fptr)
 {
 	int i;
-	mqtt3_context *context;
+	struct mosquitto *context;
 	uint16_t i16temp, slen;
 	uint32_t length;
 
 	assert(db);
-	assert(db_fd >= 0);
+	assert(db_fptr);
 
 	for(i=0; i<db->context_count; i++){
 		context = db->contexts[i];
-		if(context && context->core.clean_session == false){
-			length = htonl(2+strlen(context->core.id) + sizeof(uint16_t));
+		if(context && context->clean_session == false){
+			length = htonl(2+strlen(context->id) + sizeof(uint16_t));
 
 			i16temp = htons(DB_CHUNK_CLIENT);
-			write_e(db_fd, &i16temp, sizeof(uint16_t));
-			write_e(db_fd, &length, sizeof(uint32_t));
+			write_e(db_fptr, &i16temp, sizeof(uint16_t));
+			write_e(db_fptr, &length, sizeof(uint32_t));
 
-			slen = strlen(context->core.id);
+			slen = strlen(context->id);
 			i16temp = htons(slen);
-			write_e(db_fd, &i16temp, sizeof(uint16_t));
-			write_e(db_fd, context->core.id, slen);
-			i16temp = htons(context->core.last_mid);
-			write_e(db_fd, &i16temp, sizeof(uint16_t));
+			write_e(db_fptr, &i16temp, sizeof(uint16_t));
+			write_e(db_fptr, context->id, slen);
+			i16temp = htons(context->last_mid);
+			write_e(db_fptr, &i16temp, sizeof(uint16_t));
 
-			if(mqtt3_db_client_messages_write(db, db_fd, context)) return 1;
+			if(mqtt3_db_client_messages_write(db, db_fptr, context)) return 1;
 		}
 	}
 
 	return MOSQ_ERR_SUCCESS;
 error:
-	mqtt3_log_printf(MOSQ_LOG_ERR, "Error: %s.", strerror(errno));
+	_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: %s.", strerror(errno));
 	return 1;
 }
 
-static int _db_subs_retain_write(mosquitto_db *db, int db_fd, struct _mosquitto_subhier *node, const char *topic)
+static int _db_subs_retain_write(mosquitto_db *db, FILE *db_fptr, struct _mosquitto_subhier *node, const char *topic)
 {
 	struct _mosquitto_subhier *subhier;
 	struct _mosquitto_subleaf *sub;
@@ -267,24 +269,24 @@ static int _db_subs_retain_write(mosquitto_db *db, int db_fd, struct _mosquitto_
 
 	sub = node->subs;
 	while(sub){
-		if(sub->context->core.clean_session == false){
-			length = htonl(2+strlen(sub->context->core.id) + 2+strlen(thistopic) + sizeof(uint8_t));
+		if(sub->context->clean_session == false){
+			length = htonl(2+strlen(sub->context->id) + 2+strlen(thistopic) + sizeof(uint8_t));
 
 			i16temp = htons(DB_CHUNK_SUB);
-			write_e(db_fd, &i16temp, sizeof(uint16_t));
-			write_e(db_fd, &length, sizeof(uint32_t));
+			write_e(db_fptr, &i16temp, sizeof(uint16_t));
+			write_e(db_fptr, &length, sizeof(uint32_t));
 
-			slen = strlen(sub->context->core.id);
+			slen = strlen(sub->context->id);
 			i16temp = htons(slen);
-			write_e(db_fd, &i16temp, sizeof(uint16_t));
-			write_e(db_fd, sub->context->core.id, slen);
+			write_e(db_fptr, &i16temp, sizeof(uint16_t));
+			write_e(db_fptr, sub->context->id, slen);
 
 			slen = strlen(thistopic);
 			i16temp = htons(slen);
-			write_e(db_fd, &i16temp, sizeof(uint16_t));
-			write_e(db_fd, thistopic, slen);
+			write_e(db_fptr, &i16temp, sizeof(uint16_t));
+			write_e(db_fptr, thistopic, slen);
 
-			write_e(db_fd, &sub->qos, sizeof(uint8_t));
+			write_e(db_fptr, &sub->qos, sizeof(uint8_t));
 		}
 		sub = sub->next;
 	}
@@ -292,32 +294,32 @@ static int _db_subs_retain_write(mosquitto_db *db, int db_fd, struct _mosquitto_
 		length = htonl(sizeof(dbid_t));
 
 		i16temp = htons(DB_CHUNK_RETAIN);
-		write_e(db_fd, &i16temp, sizeof(uint16_t));
-		write_e(db_fd, &length, sizeof(uint32_t));
+		write_e(db_fptr, &i16temp, sizeof(uint16_t));
+		write_e(db_fptr, &length, sizeof(uint32_t));
 
 		i64temp = node->retained->db_id;
-		write_e(db_fd, &i64temp, sizeof(dbid_t));
+		write_e(db_fptr, &i64temp, sizeof(dbid_t));
 	}
 
 	subhier = node->children;
 	while(subhier){
-		_db_subs_retain_write(db, db_fd, subhier, thistopic);
+		_db_subs_retain_write(db, db_fptr, subhier, thistopic);
 		subhier = subhier->next;
 	}
 	_mosquitto_free(thistopic);
 	return MOSQ_ERR_SUCCESS;
 error:
-	mqtt3_log_printf(MOSQ_LOG_ERR, "Error: %s.", strerror(errno));
+	_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: %s.", strerror(errno));
 	return 1;
 }
 
-static int mqtt3_db_subs_retain_write(mosquitto_db *db, int db_fd)
+static int mqtt3_db_subs_retain_write(mosquitto_db *db, FILE *db_fptr)
 {
 	struct _mosquitto_subhier *subhier;
 
 	subhier = db->subs.children;
 	while(subhier){
-		_db_subs_retain_write(db, db_fd, subhier, "");
+		_db_subs_retain_write(db, db_fptr, subhier, "");
 		subhier = subhier->next;
 	}
 	
@@ -327,7 +329,7 @@ static int mqtt3_db_subs_retain_write(mosquitto_db *db, int db_fd)
 int mqtt3_db_backup(mosquitto_db *db, bool cleanup, bool shutdown)
 {
 	int rc = 0;
-	int db_fd;
+	FILE *db_fptr = NULL;
 	uint32_t db_version = htonl(MOSQ_DB_VERSION);
 	uint32_t crc = htonl(0);
 	dbid_t i64temp;
@@ -336,48 +338,48 @@ int mqtt3_db_backup(mosquitto_db *db, bool cleanup, bool shutdown)
 	uint8_t i8temp;
 
 	if(!db || !db->config || !db->config->persistence_filepath) return MOSQ_ERR_INVAL;
-	mqtt3_log_printf(MOSQ_LOG_INFO, "Saving in-memory database to %s.", db->config->persistence_filepath);
+	_mosquitto_log_printf(NULL, MOSQ_LOG_INFO, "Saving in-memory database to %s.", db->config->persistence_filepath);
 	if(cleanup){
 		mqtt3_db_store_clean(db);
 	}
 
-	db_fd = open(db->config->persistence_filepath, O_WRONLY|O_CREAT|O_TRUNC, S_IWUSR|S_IRUSR);
-	if(db_fd < 0){
+	db_fptr = fopen(db->config->persistence_filepath, "wb");
+	if(db_fptr == NULL){
 		goto error;
 	}
 
 	/* Header */
-	write_e(db_fd, magic, 15);
-	write_e(db_fd, &crc, sizeof(uint32_t));
-	write_e(db_fd, &db_version, sizeof(uint32_t));
+	write_e(db_fptr, magic, 15);
+	write_e(db_fptr, &crc, sizeof(uint32_t));
+	write_e(db_fptr, &db_version, sizeof(uint32_t));
 
 	/* DB config */
 	i16temp = htons(DB_CHUNK_CFG);
-	write_e(db_fd, &i16temp, sizeof(uint16_t));
+	write_e(db_fptr, &i16temp, sizeof(uint16_t));
 	/* chunk length */
 	i32temp = htonl(sizeof(dbid_t) + sizeof(uint8_t) + sizeof(uint8_t));
-	write_e(db_fd, &i32temp, sizeof(uint32_t));
+	write_e(db_fptr, &i32temp, sizeof(uint32_t));
 	/* db written at broker shutdown or not */
 	i8temp = shutdown;
-	write_e(db_fd, &i8temp, sizeof(uint8_t));
+	write_e(db_fptr, &i8temp, sizeof(uint8_t));
 	i8temp = sizeof(dbid_t);
-	write_e(db_fd, &i8temp, sizeof(uint8_t));
+	write_e(db_fptr, &i8temp, sizeof(uint8_t));
 	/* last db mid */
 	i64temp = db->last_db_id;
-	write_e(db_fd, &i64temp, sizeof(dbid_t));
+	write_e(db_fptr, &i64temp, sizeof(dbid_t));
 
-	if(mqtt3_db_message_store_write(db, db_fd)){
+	if(mqtt3_db_message_store_write(db, db_fptr)){
 		goto error;
 	}
 
-	mqtt3_db_client_write(db, db_fd);
-	mqtt3_db_subs_retain_write(db, db_fd);
+	mqtt3_db_client_write(db, db_fptr);
+	mqtt3_db_subs_retain_write(db, db_fptr);
 
-	close(db_fd);
+	fclose(db_fptr);
 	return rc;
 error:
-	mqtt3_log_printf(MOSQ_LOG_ERR, "Error: %s.", strerror(errno));
-	if(db_fd >= 0) close(db_fd);
+	_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: %s.", strerror(errno));
+	if(db_fptr) fclose(db_fptr);
 	return 1;
 }
 
@@ -385,11 +387,11 @@ static int _db_client_msg_restore(mosquitto_db *db, const char *client_id, uint1
 {
 	mosquitto_client_msg *cmsg, *tail;
 	struct mosquitto_msg_store *store;
-	mqtt3_context *context;
+	struct mosquitto *context;
 
 	cmsg = _mosquitto_calloc(1, sizeof(mosquitto_client_msg));
 	if(!cmsg){
-		mqtt3_log_printf(MOSQ_LOG_ERR, "Error: Out of memory.");
+		_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Out of memory.");
 		return MOSQ_ERR_NOMEM;
 	}
 
@@ -412,13 +414,13 @@ static int _db_client_msg_restore(mosquitto_db *db, const char *client_id, uint1
 	}
 	if(!cmsg->store){
 		_mosquitto_free(cmsg);
-		mqtt3_log_printf(MOSQ_LOG_ERR, "Error restoring persistent database, message store corrupt.");
+		_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error restoring persistent database, message store corrupt.");
 		return 1;
 	}
 	context = _db_find_or_add_context(db, client_id, 0);
 	if(!context){
 		_mosquitto_free(cmsg);
-		mqtt3_log_printf(MOSQ_LOG_ERR, "Error restoring persistent database, message store corrupt.");
+		_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error restoring persistent database, message store corrupt.");
 		return 1;
 	}
 	if(context->msgs){
@@ -435,29 +437,29 @@ static int _db_client_msg_restore(mosquitto_db *db, const char *client_id, uint1
 	return MOSQ_ERR_SUCCESS;
 }
 
-static int _db_client_chunk_restore(mosquitto_db *db, int db_fd)
+static int _db_client_chunk_restore(mosquitto_db *db, FILE *db_fptr)
 {
 	uint16_t i16temp, slen, last_mid;
 	char *client_id = NULL;
 	int rc = 0;
-	mqtt3_context *context;
+	struct mosquitto *context;
 
-	read_e(db_fd, &i16temp, sizeof(uint16_t));
+	read_e(db_fptr, &i16temp, sizeof(uint16_t));
 	slen = ntohs(i16temp);
 	if(!slen){
-		mqtt3_log_printf(MOSQ_LOG_ERR, "Error: Corrupt persistent database.");
-		close(db_fd);
+		_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Corrupt persistent database.");
+		fclose(db_fptr);
 		return 1;
 	}
 	client_id = _mosquitto_calloc(slen+1, sizeof(char));
 	if(!client_id){
-		close(db_fd);
-		mqtt3_log_printf(MOSQ_LOG_ERR, "Error: Out of memory.");
+		fclose(db_fptr);
+		_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Out of memory.");
 		return MOSQ_ERR_NOMEM;
 	}
-	read_e(db_fd, client_id, slen);
+	read_e(db_fptr, client_id, slen);
 
-	read_e(db_fd, &i16temp, sizeof(uint16_t));
+	read_e(db_fptr, &i16temp, sizeof(uint16_t));
 	last_mid = ntohs(i16temp);
 
 	context = _db_find_or_add_context(db, client_id, last_mid);
@@ -467,13 +469,13 @@ static int _db_client_chunk_restore(mosquitto_db *db, int db_fd)
 
 	return rc;
 error:
-	mqtt3_log_printf(MOSQ_LOG_ERR, "Error: %s.", strerror(errno));
-	if(db_fd >= 0) close(db_fd);
+	_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: %s.", strerror(errno));
+	if(db_fptr) fclose(db_fptr);
 	if(client_id) _mosquitto_free(client_id);
 	return 1;
 }
 
-static int _db_client_msg_chunk_restore(mosquitto_db *db, int db_fd)
+static int _db_client_msg_chunk_restore(mosquitto_db *db, FILE *db_fptr)
 {
 	dbid_t i64temp, store_id;
 	uint16_t i16temp, slen, mid;
@@ -481,45 +483,45 @@ static int _db_client_msg_chunk_restore(mosquitto_db *db, int db_fd)
 	char *client_id = NULL;
 	int rc;
 
-	read_e(db_fd, &i16temp, sizeof(uint16_t));
+	read_e(db_fptr, &i16temp, sizeof(uint16_t));
 	slen = ntohs(i16temp);
 	if(!slen){
-		mqtt3_log_printf(MOSQ_LOG_ERR, "Error: Corrupt persistent database.");
-		close(db_fd);
+		_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Corrupt persistent database.");
+		fclose(db_fptr);
 		return 1;
 	}
 	client_id = _mosquitto_calloc(slen+1, sizeof(char));
 	if(!client_id){
-		close(db_fd);
-		mqtt3_log_printf(MOSQ_LOG_ERR, "Error: Out of memory.");
+		fclose(db_fptr);
+		_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Out of memory.");
 		return MOSQ_ERR_NOMEM;
 	}
-	read_e(db_fd, client_id, slen);
+	read_e(db_fptr, client_id, slen);
 
-	read_e(db_fd, &i64temp, sizeof(dbid_t));
+	read_e(db_fptr, &i64temp, sizeof(dbid_t));
 	store_id = i64temp;
 
-	read_e(db_fd, &i16temp, sizeof(uint16_t));
+	read_e(db_fptr, &i16temp, sizeof(uint16_t));
 	mid = ntohs(i16temp);
 
-	read_e(db_fd, &qos, sizeof(uint8_t));
-	read_e(db_fd, &retain, sizeof(uint8_t));
-	read_e(db_fd, &direction, sizeof(uint8_t));
-	read_e(db_fd, &state, sizeof(uint8_t));
-	read_e(db_fd, &dup, sizeof(uint8_t));
+	read_e(db_fptr, &qos, sizeof(uint8_t));
+	read_e(db_fptr, &retain, sizeof(uint8_t));
+	read_e(db_fptr, &direction, sizeof(uint8_t));
+	read_e(db_fptr, &state, sizeof(uint8_t));
+	read_e(db_fptr, &dup, sizeof(uint8_t));
 
 	rc = _db_client_msg_restore(db, client_id, mid, qos, retain, direction, state, dup, store_id);
 	_mosquitto_free(client_id);
 
 	return rc;
 error:
-	mqtt3_log_printf(MOSQ_LOG_ERR, "Error: %s.", strerror(errno));
-	if(db_fd >= 0) close(db_fd);
+	_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: %s.", strerror(errno));
+	if(db_fptr) fclose(db_fptr);
 	if(client_id) _mosquitto_free(client_id);
 	return 1;
 }
 
-static int _db_msg_store_chunk_restore(mosquitto_db *db, int db_fd)
+static int _db_msg_store_chunk_restore(mosquitto_db *db, FILE *db_fptr)
 {
 	dbid_t i64temp, store_id;
 	uint32_t i32temp, payloadlen;
@@ -530,72 +532,72 @@ static int _db_msg_store_chunk_restore(mosquitto_db *db, int db_fd)
 	int rc = 0;
 	struct mosquitto_msg_store *stored = NULL;
 
-	read_e(db_fd, &i64temp, sizeof(dbid_t));
+	read_e(db_fptr, &i64temp, sizeof(dbid_t));
 	store_id = i64temp;
 
-	read_e(db_fd, &i16temp, sizeof(uint16_t));
+	read_e(db_fptr, &i16temp, sizeof(uint16_t));
 	slen = ntohs(i16temp);
 	if(slen){
 		source_id = _mosquitto_calloc(slen+1, sizeof(char));
 		if(!source_id){
-			close(db_fd);
-			mqtt3_log_printf(MOSQ_LOG_ERR, "Error: Out of memory.");
+			fclose(db_fptr);
+			_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Out of memory.");
 			return MOSQ_ERR_NOMEM;
 		}
-		if(read(db_fd, source_id, slen) != slen){
-			mqtt3_log_printf(MOSQ_LOG_ERR, "Error: %s.", strerror(errno));
-			close(db_fd);
+		if(fread(source_id, 1, slen, db_fptr) != slen){
+			_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: %s.", strerror(errno));
+			fclose(db_fptr);
 			_mosquitto_free(source_id);
 			return 1;
 		}
 	}
-	read_e(db_fd, &i16temp, sizeof(uint16_t));
+	read_e(db_fptr, &i16temp, sizeof(uint16_t));
 	source_mid = ntohs(i16temp);
 
-	read_e(db_fd, &i16temp, sizeof(uint16_t));
+	read_e(db_fptr, &i16temp, sizeof(uint16_t));
 	mid = ntohs(i16temp);
 
-	read_e(db_fd, &i16temp, sizeof(uint16_t));
+	read_e(db_fptr, &i16temp, sizeof(uint16_t));
 	slen = ntohs(i16temp);
 	if(slen){
 		topic = _mosquitto_calloc(slen+1, sizeof(char));
 		if(!topic){
-			close(db_fd);
+			fclose(db_fptr);
 			_mosquitto_free(source_id);
-			mqtt3_log_printf(MOSQ_LOG_ERR, "Error: Out of memory.");
+			_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Out of memory.");
 			return MOSQ_ERR_NOMEM;
 		}
-		if(read(db_fd, topic, slen) != slen){
-			mqtt3_log_printf(MOSQ_LOG_ERR, "Error: %s.", strerror(errno));
-			close(db_fd);
+		if(fread(topic, 1, slen, db_fptr) != slen){
+			_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: %s.", strerror(errno));
+			fclose(db_fptr);
 			_mosquitto_free(source_id);
 			_mosquitto_free(topic);
 			return 1;
 		}
 	}else{
-		mqtt3_log_printf(MOSQ_LOG_ERR, "Error: Invalid msg_store chunk when restoring persistent database.");
-		close(db_fd);
+		_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Invalid msg_store chunk when restoring persistent database.");
+		fclose(db_fptr);
 		_mosquitto_free(source_id);
 		return 1;
 	}
-	read_e(db_fd, &qos, sizeof(uint8_t));
-	read_e(db_fd, &retain, sizeof(uint8_t));
+	read_e(db_fptr, &qos, sizeof(uint8_t));
+	read_e(db_fptr, &retain, sizeof(uint8_t));
 	
-	read_e(db_fd, &i32temp, sizeof(uint32_t));
+	read_e(db_fptr, &i32temp, sizeof(uint32_t));
 	payloadlen = ntohl(i32temp);
 
 	if(payloadlen){
 		payload = _mosquitto_malloc(payloadlen);
 		if(!payload){
-			close(db_fd);
+			fclose(db_fptr);
 			_mosquitto_free(source_id);
 			_mosquitto_free(topic);
-			mqtt3_log_printf(MOSQ_LOG_ERR, "Error: Out of memory.");
+			_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Out of memory.");
 			return MOSQ_ERR_NOMEM;
 		}
-		if(read(db_fd, payload, payloadlen) != payloadlen){
-			mqtt3_log_printf(MOSQ_LOG_ERR, "Error: %s.", strerror(errno));
-			close(db_fd);
+		if(fread(payload, 1, payloadlen, db_fptr) != payloadlen){
+			_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: %s.", strerror(errno));
+			fclose(db_fptr);
 			_mosquitto_free(source_id);
 			_mosquitto_free(topic);
 			_mosquitto_free(payload);
@@ -610,21 +612,21 @@ static int _db_msg_store_chunk_restore(mosquitto_db *db, int db_fd)
 
 	return rc;
 error:
-	mqtt3_log_printf(MOSQ_LOG_ERR, "Error: %s.", strerror(errno));
-	if(db_fd >= 0) close(db_fd);
+	_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: %s.", strerror(errno));
+	if(db_fptr) fclose(db_fptr);
 	if(source_id) _mosquitto_free(source_id);
 	if(topic) _mosquitto_free(topic);
 	return 1;
 }
 
-static int _db_retain_chunk_restore(mosquitto_db *db, int db_fd)
+static int _db_retain_chunk_restore(mosquitto_db *db, FILE *db_fptr)
 {
 	dbid_t i64temp, store_id;
 	struct mosquitto_msg_store *store;
 
-	if(read(db_fd, &i64temp, sizeof(dbid_t)) != sizeof(dbid_t)){
-		mqtt3_log_printf(MOSQ_LOG_ERR, "Error: %s.", strerror(errno));
-		close(db_fd);
+	if(fread(&i64temp, 1, sizeof(dbid_t), db_fptr) != sizeof(dbid_t)){
+		_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: %s.", strerror(errno));
+		fclose(db_fptr);
 		return 1;
 	}
 	store_id = i64temp;
@@ -639,7 +641,7 @@ static int _db_retain_chunk_restore(mosquitto_db *db, int db_fd)
 	return MOSQ_ERR_SUCCESS;
 }
 
-static int _db_sub_chunk_restore(mosquitto_db *db, int db_fd)
+static int _db_sub_chunk_restore(mosquitto_db *db, FILE *db_fptr)
 {
 	uint16_t i16temp, slen;
 	uint8_t qos;
@@ -647,26 +649,26 @@ static int _db_sub_chunk_restore(mosquitto_db *db, int db_fd)
 	char *topic;
 	int rc = 0;
 
-	read_e(db_fd, &i16temp, sizeof(uint16_t));
+	read_e(db_fptr, &i16temp, sizeof(uint16_t));
 	slen = ntohs(i16temp);
 	client_id = _mosquitto_calloc(slen+1, sizeof(char));
 	if(!client_id){
-		close(db_fd);
-		mqtt3_log_printf(MOSQ_LOG_ERR, "Error: Out of memory.");
+		fclose(db_fptr);
+		_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Out of memory.");
 		return MOSQ_ERR_NOMEM;
 	}
-	read_e(db_fd, client_id, slen);
-	read_e(db_fd, &i16temp, sizeof(uint16_t));
+	read_e(db_fptr, client_id, slen);
+	read_e(db_fptr, &i16temp, sizeof(uint16_t));
 	slen = ntohs(i16temp);
 	topic = _mosquitto_calloc(slen+1, sizeof(char));
 	if(!topic){
-		close(db_fd);
-		mqtt3_log_printf(MOSQ_LOG_ERR, "Error: Out of memory.");
+		fclose(db_fptr);
+		_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Out of memory.");
 		_mosquitto_free(client_id);
 		return MOSQ_ERR_NOMEM;
 	}
-	read_e(db_fd, topic, slen);
-	read_e(db_fd, &qos, sizeof(uint8_t));
+	read_e(db_fptr, topic, slen);
+	read_e(db_fptr, &qos, sizeof(uint8_t));
 	if(_db_restore_sub(db, client_id, topic, qos)){
 		rc = 1;
 	}
@@ -675,14 +677,14 @@ static int _db_sub_chunk_restore(mosquitto_db *db, int db_fd)
 
 	return rc;
 error:
-	mqtt3_log_printf(MOSQ_LOG_ERR, "Error: %s.", strerror(errno));
-	if(db_fd >= 0) close(db_fd);
+	_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: %s.", strerror(errno));
+	if(db_fptr) fclose(db_fptr);
 	return 1;
 }
 
 int mqtt3_db_restore(mosquitto_db *db)
 {
-	int fd;
+	FILE *fptr;
 	char header[15];
 	int rc = 0;
 	uint32_t crc, db_version;
@@ -696,85 +698,85 @@ int mqtt3_db_restore(mosquitto_db *db)
 	assert(db->config);
 	assert(db->config->persistence_filepath);
 
-	fd = open(db->config->persistence_filepath, O_RDONLY);
-	if(fd < 0) return MOSQ_ERR_SUCCESS;
-	read_e(fd, &header, 15);
+	fptr = fopen(db->config->persistence_filepath, "rb");
+	if(fptr == NULL) return MOSQ_ERR_SUCCESS;
+	read_e(fptr, &header, 15);
 	if(!memcmp(header, magic, 15)){
 		// Restore DB as normal
-		read_e(fd, &crc, sizeof(uint32_t));
-		read_e(fd, &i32temp, sizeof(uint32_t));
+		read_e(fptr, &crc, sizeof(uint32_t));
+		read_e(fptr, &i32temp, sizeof(uint32_t));
 		db_version = ntohl(i32temp);
 		/* IMPORTANT - this is where compatibility checks are made.
 		 * Is your DB change still compatible with previous versions?
 		 */
 		if(db_version > MOSQ_DB_VERSION && db_version != 0){
-			close(fd);
-			mqtt3_log_printf(MOSQ_LOG_ERR, "Error: Unsupported persistent database format version %d (need version %d).", db_version, MOSQ_DB_VERSION);
+			fclose(fptr);
+			_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Unsupported persistent database format version %d (need version %d).", db_version, MOSQ_DB_VERSION);
 			return 1;
 		}
 
-		while(rlen = read(fd, &i16temp, sizeof(uint16_t)), rlen == sizeof(uint16_t)){
+		while(rlen = fread(&i16temp, 1, sizeof(uint16_t), fptr), rlen == sizeof(uint16_t)){
 			chunk = ntohs(i16temp);
-			read_e(fd, &i32temp, sizeof(uint32_t));
+			read_e(fptr, &i32temp, sizeof(uint32_t));
 			length = ntohl(i32temp);
 			switch(chunk){
 				case DB_CHUNK_CFG:
-					read_e(fd, &i8temp, sizeof(uint8_t)); // shutdown
-					read_e(fd, &i8temp, sizeof(uint8_t)); // sizeof(dbid_t)
+					read_e(fptr, &i8temp, sizeof(uint8_t)); // shutdown
+					read_e(fptr, &i8temp, sizeof(uint8_t)); // sizeof(dbid_t)
 					if(i8temp != sizeof(dbid_t)){
-						mqtt3_log_printf(MOSQ_LOG_ERR, "Error: Incompatible database configuration (dbid size is %d bytes, expected %d)",
+						_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Incompatible database configuration (dbid size is %d bytes, expected %d)",
 								i8temp, sizeof(dbid_t));
-						close(fd);
+						fclose(fptr);
 						return 1;
 					}
-					read_e(fd, &i64temp, sizeof(dbid_t));
+					read_e(fptr, &i64temp, sizeof(dbid_t));
 					db->last_db_id = i64temp;
 					break;
 
 				case DB_CHUNK_MSG_STORE:
-					if(_db_msg_store_chunk_restore(db, fd)) return 1;
+					if(_db_msg_store_chunk_restore(db, fptr)) return 1;
 					break;
 
 				case DB_CHUNK_CLIENT_MSG:
-					if(_db_client_msg_chunk_restore(db, fd)) return 1;
+					if(_db_client_msg_chunk_restore(db, fptr)) return 1;
 					break;
 
 				case DB_CHUNK_RETAIN:
-					if(_db_retain_chunk_restore(db, fd)) return 1;
+					if(_db_retain_chunk_restore(db, fptr)) return 1;
 					break;
 
 				case DB_CHUNK_SUB:
-					if(_db_sub_chunk_restore(db, fd)) return 1;
+					if(_db_sub_chunk_restore(db, fptr)) return 1;
 					break;
 
 				case DB_CHUNK_CLIENT:
-					if(_db_client_chunk_restore(db, fd)) return 1;
+					if(_db_client_chunk_restore(db, fptr)) return 1;
 					break;
 
 				default:
-					mqtt3_log_printf(MOSQ_LOG_WARNING, "Warning: Unsupported chunk \"%d\" in persistent database file. Ignoring.", chunk);
-					lseek(fd, length, SEEK_CUR);
+					_mosquitto_log_printf(NULL, MOSQ_LOG_WARNING, "Warning: Unsupported chunk \"%d\" in persistent database file. Ignoring.", chunk);
+					fseek(fptr, length, SEEK_CUR);
 					break;
 			}
 		}
 		if(rlen < 0) goto error;
 	}else{
-		mqtt3_log_printf(MOSQ_LOG_ERR, "Error: Unable to restore persistent database. Unrecognised file format.");
+		_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Unable to restore persistent database. Unrecognised file format.");
 		rc = 1;
 	}
 
-	close(fd);
+	fclose(fptr);
 
 	return rc;
 error:
-	mqtt3_log_printf(MOSQ_LOG_ERR, "Error: %s.", strerror(errno));
-	if(fd >= 0) close(fd);
+	_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: %s.", strerror(errno));
+	if(fptr) fclose(fptr);
 	return 1;
 }
 
 static int _db_restore_sub(mosquitto_db *db, const char *client_id, const char *sub, int qos)
 {
-	mqtt3_context *context;
+	struct mosquitto *context;
 
 	assert(db);
 	assert(client_id);
