@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2009-2011 Roger Light <roger@atchoo.org>
+Copyright (c) 2009-2012 Roger Light <roger@atchoo.org>
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -107,7 +107,7 @@ int mqtt3_handle_publish(mosquitto_db *db, struct mosquitto *context)
 	if(!strlen(topic)){
 		return 1;
 	}
-	if(_mosquitto_wildcard_check(topic)){
+	if(_mosquitto_topic_wildcard_len_check(topic) != MOSQ_ERR_SUCCESS){
 		/* Invalid publish topic, just swallow it. */
 		_mosquitto_free(topic);
 		return MOSQ_ERR_SUCCESS;
@@ -177,7 +177,7 @@ int mqtt3_handle_publish(mosquitto_db *db, struct mosquitto *context)
 			break;
 		case 2:
 			if(!dup){
-				res = mqtt3_db_message_insert(context, mid, mosq_md_in, qos, retain, stored);
+				res = mqtt3_db_message_insert(db, context, mid, mosq_md_in, qos, retain, stored);
 			}else{
 				res = 0;
 			}
