@@ -27,7 +27,7 @@ ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 */
 
-#ifdef WIN32
+#if defined(WIN32) || defined(__CYGWIN__)
 
 #include <windows.h>
 
